from enum import Enum

from typing_extensions import Self

from frogml_storage.constants import (
    BODY_PART_DATASET_MANIFEST_STREAM,
    BODY_PART_MODEL_MANIFEST_STREAM,
    DATASET_METADATA_FILE_NAME,
    DATASET_UI_DIRECTORY,
    MODEL_METADATA_FILE_NAME,
    MODEL_UI_DIRECTORY,
    ROOT_FROGML_DATASET_UI_DIRECTORY,
    ROOT_FROGML_MODEL_UI_DIRECTORY,
)


# noinspection PyEnum
class FrogMLEntityTypeInfo(Enum):
    MODEL = (
        MODEL_UI_DIRECTORY,
        ROOT_FROGML_MODEL_UI_DIRECTORY,
        MODEL_METADATA_FILE_NAME,
        BODY_PART_MODEL_MANIFEST_STREAM,
    )
    DATASET = (
        DATASET_UI_DIRECTORY,
        ROOT_FROGML_DATASET_UI_DIRECTORY,
        DATASET_METADATA_FILE_NAME,
        BODY_PART_DATASET_MANIFEST_STREAM,
    )

    def __init__(
        self: Self,
        entity_type: str,
        folder_name: str,
        metadata_file_name: str,
        body_part_stream: str,
    ):
        self.entity_type: str = entity_type
        self.folder_name: str = folder_name
        self.metadata_file_name: str = metadata_file_name
        self.body_part_stream: str = body_part_stream

    @classmethod
    def from_string(cls, entity_type_string: str) -> "FrogMLEntityTypeInfo":
        for entity_type in cls:
            if entity_type.entity_type.lower() == entity_type_string.lower():
                return entity_type

        raise ValueError(f"No enum constant found for entityType: {entity_type_string}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/features_operator/v2/features_operator_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.features_operator.v2 import features_operator_pb2 as qwak_dot_features__operator_dot_v2_dot_features__operator__pb2
from _qwak_proto.qwak.feature_store.features import feature_set_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__pb2
from _qwak_proto.qwak.feature_store.sources import data_source_pb2 as qwak_dot_feature__store_dot_sources_dot_data__source__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9qwak/features_operator/v2/features_operator_service.proto\x12\x19qwak.features_operator.v2\x1a\x31qwak/features_operator/v2/features_operator.proto\x1a-qwak/feature_store/features/feature_set.proto\x1a,qwak/feature_store/sources/data_source.proto\"\x88\x01\n\x19ValidateDataSourceRequest\x12\x44\n\x10\x64\x61ta_source_spec\x18\x01 \x01(\x0b\x32*.qwak.feature.store.sources.DataSourceSpec\x12\x13\n\x0bnum_samples\x18\x02 \x01(\x05\x12\x10\n\x08zip_path\x18\x03 \x01(\t\"\x89\x01\n\x19ValidateFeatureSetRequest\x12\x45\n\x10\x66\x65\x61ture_set_spec\x18\x01 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetSpec\x12\x10\n\x08zip_path\x18\x02 \x01(\t\x12\x13\n\x0bnum_samples\x18\x03 \x01(\x05\"\xc0\x01\n\x12ValidationResponse\x12P\n\x10success_response\x18\x01 \x01(\x0b\x32\x34.qwak.features_operator.v2.ValidationSuccessResponseH\x00\x12P\n\x10\x66\x61ilure_response\x18\x02 \x01(\x0b\x32\x34.qwak.features_operator.v2.ValidationFailureResponseH\x00\x42\x06\n\x04type2\x8f\x02\n\x17\x46\x65\x61turesOperatorService\x12y\n\x12ValidateDataSource\x12\x34.qwak.features_operator.v2.ValidateDataSourceRequest\x1a-.qwak.features_operator.v2.ValidationResponse\x12y\n\x12ValidateFeatureSet\x12\x34.qwak.features_operator.v2.ValidateFeatureSetRequest\x1a-.qwak.features_operator.v2.ValidationResponseB(\n$com.qwak.ai.features.operator.api.v2P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.features_operator.v2.features_operator_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.qwak.ai.features.operator.api.v2P\001'
  _globals['_VALIDATEDATASOURCEREQUEST']._serialized_start=233
  _globals['_VALIDATEDATASOURCEREQUEST']._serialized_end=369
  _globals['_VALIDATEFEATURESETREQUEST']._serialized_start=372
  _globals['_VALIDATEFEATURESETREQUEST']._serialized_end=509
  _globals['_VALIDATIONRESPONSE']._serialized_start=512
  _globals['_VALIDATIONRESPONSE']._serialized_end=704
  _globals['_FEATURESOPERATORSERVICE']._serialized_start=707
  _globals['_FEATURESOPERATORSERVICE']._serialized_end=978
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.admiral.secret.v0.secret_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SetSystemSecretRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> qwak.admiral.secret.v0.secret_pb2.SetSystemEnvironmentSecretOptions: ...
    def __init__(
        self,
        *,
        options: qwak.admiral.secret.v0.secret_pb2.SetSystemEnvironmentSecretOptions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___SetSystemSecretRequest = SetSystemSecretRequest

class SetSystemSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SetSystemSecretResponse = SetSystemSecretResponse

class DeleteSystemSecretRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDENTIFIER_FIELD_NUMBER: builtins.int
    @property
    def identifier(self) -> qwak.admiral.secret.v0.secret_pb2.EnvironmentSecretIdentifier:
        """Secret identifier"""
    def __init__(
        self,
        *,
        identifier: qwak.admiral.secret.v0.secret_pb2.EnvironmentSecretIdentifier | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["identifier", b"identifier"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["identifier", b"identifier"]) -> None: ...

global___DeleteSystemSecretRequest = DeleteSystemSecretRequest

class DeleteSystemSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteSystemSecretResponse = DeleteSystemSecretResponse

class GetSystemSecretRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDENTIFIER_FIELD_NUMBER: builtins.int
    @property
    def identifier(self) -> qwak.admiral.secret.v0.secret_pb2.EnvironmentSecretIdentifier:
        """Secret identifier"""
    def __init__(
        self,
        *,
        identifier: qwak.admiral.secret.v0.secret_pb2.EnvironmentSecretIdentifier | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["identifier", b"identifier"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["identifier", b"identifier"]) -> None: ...

global___GetSystemSecretRequest = GetSystemSecretRequest

class GetSystemSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_DEFINITION_FIELD_NUMBER: builtins.int
    @property
    def secret_definition(self) -> qwak.admiral.secret.v0.secret_pb2.SystemSecretDefinition:
        """Secret definition"""
    def __init__(
        self,
        *,
        secret_definition: qwak.admiral.secret.v0.secret_pb2.SystemSecretDefinition | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["secret_definition", b"secret_definition"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["secret_definition", b"secret_definition"]) -> None: ...

global___GetSystemSecretResponse = GetSystemSecretResponse



from typing import Optional

from _qwak_proto.qwak.feature_store.entities.entity_pb2 import EntitySpec
from qwak.clients.feature_store import FeatureRegistryClient


class Entity:
    """
    An entity is an object which features sets can be associated with.

    Features created via the feature store are tied to a specific business entity.
    For example registered users, transactions, or merchants.

    Example of an Entity definition:

    .. code-block:: python

        from qwak.feature_store import Entity

        customer = Entity(
            name='organization_id',
            description='An organization which is a customer of our platform',
        )
    """

    def __init__(self, name: str, description: str, key: Optional[str] = None):
        """
        Create a new Entity

        :param name: The name of the entity, must be unique
        :param description: Short human-readable description of the entity
        :param key: The actual key this entity is referring to. Used when user provides a key instead of
                    entity configuration. Currently only a single key is supported.
        """
        self.name = name
        self.description = description
        self.key: Optional[str] = key

    def _to_proto(self):
        return EntitySpec(
            name=self.name,
            keys=[self.name if not self.key else self.key],
            description=self.description,
            value_type=1,  # string
        )

    @classmethod
    def _from_proto(cls, proto):
        entity_spec: EntitySpec = proto.entity_spec

        # entity spec keys is a repeated str but only single key is supported
        keys_repr: str = ",".join(entity_spec.keys)

        return cls(
            name=entity_spec.name,
            description=entity_spec.description,
            key=keys_repr,
        )

    def register(self):
        """
        Explicitly register this entity to Qwak's Feature Store
        """

        registry = FeatureRegistryClient()
        existing_entity = registry.get_entity_by_name(self.name)
        if existing_entity:
            registry.update_entity(
                existing_entity.entity.entity_definition.entity_id,
                self._to_proto(),
            )
        else:
            registry.create_entity(self._to_proto())



from typing import Optional

from frogml_storage.constants import (
    FROG_ML_CONFIG_ACCESS_TOKEN,
    FROG_ML_CONFIG_ARTIFACTORY_URL,
    FROG_ML_CONFIG_PASSWORD,
    FROG_ML_CONFIG_USER,
    SERVER_ID,
)


class AuthConfig(object):
    def __init__(
        self,
        artifactory_url: Optional[str] = None,
        access_token: Optional[str] = None,
        user: Optional[str] = None,
        password: Optional[str] = None,
        server_id: Optional[str] = None,
    ) -> None:
        self.artifactory_url = artifactory_url
        self.user = user
        self.password = password
        self.access_token = access_token
        self.server_id = server_id

    def to_json(self):
        include_keys = [
            FROG_ML_CONFIG_ARTIFACTORY_URL,
            FROG_ML_CONFIG_USER,
            FROG_ML_CONFIG_PASSWORD,
            FROG_ML_CONFIG_ACCESS_TOKEN,
            SERVER_ID,
        ]
        return {
            key: getattr(self, key)
            for key in include_keys
            if getattr(self, key) is not None
        }

    @classmethod
    def by_access_token(cls, artifactory_url: str, access_token: str) -> "AuthConfig":
        return cls(
            artifactory_url=artifactory_url,
            access_token=access_token,
        )

    @classmethod
    def by_basic_auth(
        cls, artifactory_url: str, user: str, password: str
    ) -> "AuthConfig":
        return cls(
            artifactory_url=artifactory_url,
            user=user,
            password=password,
        )

    @classmethod
    def by_server_id(cls, server_id: str) -> "AuthConfig":
        return cls(server_id=server_id)

    @classmethod
    def from_dict(cls, data: dict) -> "AuthConfig":
        return cls(
            artifactory_url=data.get(FROG_ML_CONFIG_ARTIFACTORY_URL),
            access_token=data.get(FROG_ML_CONFIG_ACCESS_TOKEN),
            server_id=data.get(SERVER_ID),
            user=data.get(FROG_ML_CONFIG_USER),
            password=data.get(FROG_ML_CONFIG_PASSWORD),
        )



import json

TF_B64_KEY = "b64"


class B64JsonEncoder(json.JSONEncoder):
    def default(self, o):
        import base64

        if isinstance(o, bytes):
            try:
                return o.decode("utf-8")
            except UnicodeDecodeError:
                return {TF_B64_KEY: base64.b64encode(o).decode("utf-8")}

        try:
            return super(B64JsonEncoder, self).default(o)
        except (TypeError, OverflowError):
            return {"unknown_obj": str(o)}


class NumpyJsonEncoder(B64JsonEncoder):
    def default(self, o):
        import numpy as np

        if isinstance(o, np.generic):
            return o.item()

        if isinstance(o, np.ndarray):
            return o.tolist()

        return super(NumpyJsonEncoder, self).default(o)


class TfTensorJsonEncoder(NumpyJsonEncoder):
    def default(self, o):
        import tensorflow as tf

        # Tensor -> ndarray or object
        if isinstance(o, tf.Tensor):
            if tf.__version__.startswith("1."):
                with tf.compat.v1.Session():
                    return o.numpy()
            else:
                return o.numpy()
        return super(TfTensorJsonEncoder, self).default(o)



from qwak.inner.build_config.build_config_v1 import BuildConfigV1
from qwak.inner.build_logic.interface.build_phase import BuildPhase
from qwak.inner.build_logic.phases.phase_010_fetch_model.fetch_model_step import (
    FetchModelStep,
)
from qwak.inner.build_logic.phases.phase_010_fetch_model.post_fetch_validation_step import (
    PostFetchValidationStep,
)
from qwak.inner.build_logic.phases.phase_010_fetch_model.pre_fetch_validation_step import (
    PreFetchValidationStep,
)
from qwak.inner.build_logic.phases.phase_010_fetch_model.set_version_step import (
    SetVersionStep,
)
from qwak.inner.build_logic.phases.phase_020_remote_register_qwak_build.cleanup_step import (
    CleanupStep,
)
from qwak.inner.build_logic.phases.phase_020_remote_register_qwak_build.start_remote_build_step import (
    StartRemoteBuildStep,
)
from qwak.inner.build_logic.phases.phase_020_remote_register_qwak_build.upload_step import (
    UploadStep,
)
from qwak.inner.build_logic.phases.phases_pipeline import PhasesPipeline
from qwak.inner.build_logic.trigger_build_context import TriggerBuildContext

FETCHING_MODEL_CODE_PHASE = "FETCHING_MODEL_CODE"
REGISTERING_QWAK_BUILD_PHASE = "REGISTERING_QWAK_BUILD"


def get_trigger_build_api_steps(
    config: BuildConfigV1, context: TriggerBuildContext
) -> PhasesPipeline:
    steps_root = PhasesPipeline(config=config, context=context)
    steps_root.add_phase(
        steps=[
            SetVersionStep(),
            PreFetchValidationStep(),
            FetchModelStep(),
            PostFetchValidationStep(),
        ],
        build_phase=BuildPhase(phase_id=FETCHING_MODEL_CODE_PHASE),
    )
    steps_root.add_phase(
        steps=[
            UploadStep(),
            StartRemoteBuildStep(),
            CleanupStep(),
        ],
        build_phase=BuildPhase(phase_id=REGISTERING_QWAK_BUILD_PHASE),
    )

    return steps_root



from datetime import datetime
from typing import Any

from _qwak_proto.qwak.vectors.v1.filters_pb2 import AtomicLiteral as ProtoAtomicLiteral
from qwak.utils.datetime_utils import datetime_to_pts


def transform(value: Any) -> ProtoAtomicLiteral:
    if isinstance(value, bool):
        return ProtoAtomicLiteral(bool_literal=value)
    elif isinstance(value, str):
        return ProtoAtomicLiteral(string_literal=value)
    elif isinstance(value, int):
        return ProtoAtomicLiteral(int_literal=value)
    elif isinstance(value, float):
        return ProtoAtomicLiteral(double_literal=value)
    elif isinstance(value, datetime):
        # Assuming that timestamp is a datetime
        return ProtoAtomicLiteral(timestamp_literal=datetime_to_pts(value))
    else:
        raise ValueError(f"Unsupported data type: {type(value)}")



import os
from typing import Optional, Tuple

import requests
from requests.adapters import HTTPAdapter
from urllib3 import Retry

import frogml_storage
from frogml_storage.logging import logger


class HTTPClient:

    def __init__(
        self, auth: Tuple[str, str], session: Optional[requests.Session] = None
    ):
        self.auth = auth
        # add default headers
        if session is None:
            self.session = self._create_session()
        self._add_default_headers()
        self.timeout = os.getenv("JFML_TIMEOUT", default=30)

    @staticmethod
    def _create_session():
        session = requests.Session()
        adapter = HTTPAdapter(
            max_retries=RetryWithLog(
                total=5, backoff_factor=0.5, status_forcelist=[500, 502, 503, 504]
            )
        )
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        return session

    def post(self, url, data=None, params=None):
        return self.session.post(
            url, auth=self.auth, timeout=self.timeout, data=data, params=params
        )

    def get(self, url, params=None, stream=False):
        return self.session.get(url, auth=self.auth, params=params, stream=stream)

    def put(self, url, payload=None, files=None, stream=False, headers=None, json=None):
        return self.session.request(
            method="PUT",
            url=url,
            data=payload,
            auth=self.auth,
            files=files,
            stream=stream,
            timeout=self.timeout,
            headers=headers,
            json=json,
        )

    def delete(self, url):
        return self.session.request(
            method="DELETE",
            url=url,
            auth=self.auth,
            timeout=self.timeout,
        )

    def head(self, url, params=None, stream=False):
        return self.session.head(url, auth=self.auth, params=params, stream=stream)

    def _add_default_headers(self):
        self.session.headers.update(
            {"User-Agent": "frogml-sdk-python/{}".format(frogml_storage.__version__)}
        )


class RetryWithLog(Retry):
    """
    Adding extra logs before making a retry request
    """

    def __init__(self, *args, **kwargs):
        history = kwargs.get("history")
        if history is not None:
            logger.debug(f"Error: ${history[-1].error}\nretrying...")
        super().__init__(*args, **kwargs)



from .quiet_error import QuietError


class QwakLoadModelFailedException(QuietError):
    """
    Raises when `load_model()` raises an exception
    """

    def __init__(self, exception):
        self.message = f"Failed to load model from load_model(), Error: {exception}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/automation/v1/trigger.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.automation.v1 import common_pb2 as qwak_dot_automation_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n qwak/automation/v1/trigger.proto\x12\x12qwak.automation.v1\x1a\x1fqwak/automation/v1/common.proto\"\x9e\x02\n\x07Trigger\x12\x41\n\x11scheduled_trigger\x18\x01 \x01(\x0b\x32$.qwak.automation.v1.ScheduledTriggerH\x00\x12\x46\n\x14metric_based_trigger\x18\x02 \x01(\x0b\x32&.qwak.automation.v1.MetricBasedTriggerH\x00\x12\x37\n\x0cnone_trigger\x18\x03 \x01(\x0b\x32\x1f.qwak.automation.v1.NoneTriggerH\x00\x12\x44\n\x13on_boarding_trigger\x18\x04 \x01(\x0b\x32%.qwak.automation.v1.OnBoardingTriggerH\x00\x42\t\n\x07trigger\"B\n\x10ScheduledTrigger\x12\x0e\n\x04\x63ron\x18\x01 \x01(\tH\x00\x12\x12\n\x08interval\x18\x02 \x01(\tH\x00\x42\n\n\x08schedule\"\xc3\x01\n\x12MetricBasedTrigger\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\x06metric\x18\x02 \x01(\x0b\x32\x1a.qwak.automation.v1.Metric\x12I\n\x13threshold_direction\x18\x03 \x01(\x0e\x32,.qwak.automation.v1.MetricThresholdDirection\x12\x11\n\tthreshold\x18\x04 \x01(\t\x12\x15\n\roverride_cron\x18\x05 \x01(\t\"\r\n\x0bNoneTrigger\"\x13\n\x11OnBoardingTriggerB!\n\x1d\x63om.qwak.ai.automation.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.automation.v1.trigger_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.automation.api.v1P\001'
  _globals['_TRIGGER']._serialized_start=90
  _globals['_TRIGGER']._serialized_end=376
  _globals['_SCHEDULEDTRIGGER']._serialized_start=378
  _globals['_SCHEDULEDTRIGGER']._serialized_end=444
  _globals['_METRICBASEDTRIGGER']._serialized_start=447
  _globals['_METRICBASEDTRIGGER']._serialized_end=642
  _globals['_NONETRIGGER']._serialized_start=644
  _globals['_NONETRIGGER']._serialized_end=657
  _globals['_ONBOARDINGTRIGGER']._serialized_start=659
  _globals['_ONBOARDINGTRIGGER']._serialized_end=678
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.vectors.v1 import vector_service_pb2 as qwak_dot_vectors_dot_v1_dot_vector__service__pb2


class VectorServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SearchSimilarVectors = channel.unary_unary(
                '/qwak.vector.store.serving.v1.VectorService/SearchSimilarVectors',
                request_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsRequest.SerializeToString,
                response_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsResponse.FromString,
                )
        self.GetVectorSample = channel.unary_unary(
                '/qwak.vector.store.serving.v1.VectorService/GetVectorSample',
                request_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleRequest.SerializeToString,
                response_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleResponse.FromString,
                )
        self.FetchVector = channel.unary_unary(
                '/qwak.vector.store.serving.v1.VectorService/FetchVector',
                request_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorRequest.SerializeToString,
                response_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorResponse.FromString,
                )
        self.UpsertVectors = channel.unary_unary(
                '/qwak.vector.store.serving.v1.VectorService/UpsertVectors',
                request_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsRequest.SerializeToString,
                response_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsResponse.FromString,
                )
        self.DeleteVectors = channel.unary_unary(
                '/qwak.vector.store.serving.v1.VectorService/DeleteVectors',
                request_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsRequest.SerializeToString,
                response_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsResponse.FromString,
                )


class VectorServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SearchSimilarVectors(self, request, context):
        """Search for similar vectors
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetVectorSample(self, request, context):
        """Get vector samples
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchVector(self, request, context):
        """Fetch a specific vector by ID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpsertVectors(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteVectors(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_VectorServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SearchSimilarVectors': grpc.unary_unary_rpc_method_handler(
                    servicer.SearchSimilarVectors,
                    request_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsRequest.FromString,
                    response_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsResponse.SerializeToString,
            ),
            'GetVectorSample': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVectorSample,
                    request_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleRequest.FromString,
                    response_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleResponse.SerializeToString,
            ),
            'FetchVector': grpc.unary_unary_rpc_method_handler(
                    servicer.FetchVector,
                    request_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorRequest.FromString,
                    response_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorResponse.SerializeToString,
            ),
            'UpsertVectors': grpc.unary_unary_rpc_method_handler(
                    servicer.UpsertVectors,
                    request_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsRequest.FromString,
                    response_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsResponse.SerializeToString,
            ),
            'DeleteVectors': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteVectors,
                    request_deserializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsRequest.FromString,
                    response_serializer=qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.vector.store.serving.v1.VectorService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class VectorService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SearchSimilarVectors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.vector.store.serving.v1.VectorService/SearchSimilarVectors',
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsRequest.SerializeToString,
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.SearchSimilarVectorsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetVectorSample(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.vector.store.serving.v1.VectorService/GetVectorSample',
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleRequest.SerializeToString,
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.GetVectorSampleResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FetchVector(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.vector.store.serving.v1.VectorService/FetchVector',
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorRequest.SerializeToString,
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.FetchVectorResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpsertVectors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.vector.store.serving.v1.VectorService/UpsertVectors',
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsRequest.SerializeToString,
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.UpsertVectorsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteVectors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.vector.store.serving.v1.VectorService/DeleteVectors',
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsRequest.SerializeToString,
            qwak_dot_vectors_dot_v1_dot_vector__service__pb2.DeleteVectorsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import uuid
from datetime import datetime

import grpc
from _qwak_proto.qwak.vectors.v1.collection.collection_pb2 import (
    COLLECTION_STATUS_CREATE_REQUESTED,
    QwakMetadata,
    VectorCollection,
    VectorCollectionDefinition,
)
from _qwak_proto.qwak.vectors.v1.collection.collection_service_pb2 import (
    CreateCollectionResponse,
    DeleteCollectionByIdResponse,
    DeleteCollectionByNameResponse,
    GetCollectionByIdResponse,
    GetCollectionByNameResponse,
    ListCollectionsResponse,
)
from _qwak_proto.qwak.vectors.v1.collection.collection_service_pb2_grpc import (
    VectorCollectionServiceServicer,
)
from google.protobuf.timestamp_pb2 import Timestamp


class VectorCollectionManagementServiceMock(VectorCollectionServiceServicer):
    def __init__(self):
        self._collections_spec_by_ids = {}
        self._collections_spec_by_name = {}

    def reset_collections(self):
        self._collections_spec_by_ids.clear()
        self._collections_spec_by_name.clear()

    def CreateCollection(self, request, context):
        timestamp = Timestamp()
        timestamp.FromDatetime(datetime.now())

        collection_id = str(uuid.uuid4())
        vector_collection = VectorCollection(
            metadata=QwakMetadata(
                created_at=timestamp,
                created_by="it@qwak.com",
                last_modified_at=timestamp,
                last_modified_by="it@qwak.com",
            ),
            definition=VectorCollectionDefinition(
                id=collection_id,
                collection_spec=request.collection_spec,
            ),
            status=COLLECTION_STATUS_CREATE_REQUESTED,
        )

        self._collections_spec_by_ids[collection_id] = vector_collection
        self._collections_spec_by_name[request.collection_spec.name] = vector_collection
        return CreateCollectionResponse(vector_collection=vector_collection)

    def GetCollectionById(self, request, context):
        if request.id in self._collections_spec_by_ids:
            return GetCollectionByIdResponse(
                vector_collection=self._collections_spec_by_ids[request.id]
            )

        context.set_details(f"Collection ID {request.id} doesn't exist'")
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def GetCollectionByName(self, request, context):
        if request.name in self._collections_spec_by_name:
            return GetCollectionByNameResponse(
                vector_collection=self._collections_spec_by_name[request.name]
            )

        context.set_details(f"Collection name {request.name} doesn't exist'")
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def DeleteCollectionById(self, request, context):
        if request.id in self._collections_spec_by_ids:
            self._collections_spec_by_ids.pop(request.id)
            return DeleteCollectionByIdResponse()

        context.set_details(f"Collection ID {request.id} doesn't exist'")
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def DeleteCollectionByName(self, request, context):
        if request.name in self._collections_spec_by_name:
            self._collections_spec_by_name.pop(request.name)
            return DeleteCollectionByNameResponse()

        context.set_details(f"Collection name {request.name} doesn't exist'")
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def ListCollections(self, request, context):
        return ListCollectionsResponse(
            vector_collections=[
                collection for collection in self._collections_spec_by_ids.values()
            ]
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.feature_store.entities import entity_service_pb2 as qwak_dot_feature__store_dot_entities_dot_entity__service__pb2


class EntityServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateEntity = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/CreateEntity',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityResponse.FromString,
                )
        self.UpdateEntity = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/UpdateEntity',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityResponse.FromString,
                )
        self.DeleteEntity = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/DeleteEntity',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityResponse.FromString,
                )
        self.GetEntityById = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/GetEntityById',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdResponse.FromString,
                )
        self.GetEntityByName = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/GetEntityByName',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameResponse.FromString,
                )
        self.ListEntities = channel.unary_unary(
                '/qwak.feature.store.entities.EntityService/ListEntities',
                request_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesRequest.SerializeToString,
                response_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesResponse.FromString,
                )


class EntityServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateEntity(self, request, context):
        """Create an entity in the registry
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateEntity(self, request, context):
        """Update an existing entity
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEntity(self, request, context):
        """Delete an entity
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEntityById(self, request, context):
        """Get an entity by name
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEntityByName(self, request, context):
        """Get an entity by ID
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListEntities(self, request, context):
        """List all entities
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EntityServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateEntity,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityResponse.SerializeToString,
            ),
            'UpdateEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateEntity,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityResponse.SerializeToString,
            ),
            'DeleteEntity': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEntity,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityResponse.SerializeToString,
            ),
            'GetEntityById': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEntityById,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdResponse.SerializeToString,
            ),
            'GetEntityByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEntityByName,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameResponse.SerializeToString,
            ),
            'ListEntities': grpc.unary_unary_rpc_method_handler(
                    servicer.ListEntities,
                    request_deserializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesRequest.FromString,
                    response_serializer=qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.feature.store.entities.EntityService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class EntityService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/CreateEntity',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.CreateEntityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/UpdateEntity',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.UpdateEntityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEntity(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/DeleteEntity',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.DeleteEntityResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEntityById(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/GetEntityById',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEntityByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/GetEntityByName',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.GetEntityByNameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListEntities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.entities.EntityService/ListEntities',
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesRequest.SerializeToString,
            qwak_dot_feature__store_dot_entities_dot_entity__service__pb2.ListEntitiesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/reports/report.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'qwak/feature_store/reports/report.proto\x12\x17qwak.feature.store.jobs\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd4\x01\n\x18\x42\x61tchJobIngestionDetails\x12\x38\n\x14\x62\x61tch_execution_date\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12?\n\x1bingestion_window_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12=\n\x19ingestion_window_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x1e\n\x1cStreamingJobIngestionDetails\")\n\'StreamingAggregationJobIngestionDetails\"\xec\x04\n\tJobReport\x12T\n\x17\x62\x61tch_ingestion_details\x18\x01 \x01(\x0b\x32\x31.qwak.feature.store.jobs.BatchJobIngestionDetailsH\x00\x12\\\n\x1bstreaming_ingestion_details\x18\x02 \x01(\x0b\x32\x35.qwak.feature.store.jobs.StreamingJobIngestionDetailsH\x00\x12l\n streaming_aggr_ingestion_details\x18\x03 \x01(\x0b\x32@.qwak.feature.store.jobs.StreamingAggregationJobIngestionDetailsH\x00\x12\x1c\n\x14num_affected_records\x18\x04 \x01(\x03\x12\x32\n\x0erun_start_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0crun_end_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0c\x65xecution_id\x18\x07 \x01(\x03\x12\x16\n\x0e\x65nvironment_id\x18\x08 \x01(\t\x12\x15\n\rfeatureset_id\x18\t \x01(\t\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\n \x01(\t\x12\x32\n\x06status\x18\x0b \x01(\x0e\x32\".qwak.feature.store.jobs.JobStatus\x12\x12\n\ntry_number\x18\x0c \x01(\x05\x42\x13\n\x11ingestion_details*m\n\tJobStatus\x12\x16\n\x12JOB_STATUS_INVALID\x10\x00\x12\x16\n\x12JOB_STATUS_SUCCESS\x10\x01\x12\x16\n\x12JOB_STATUS_FAILURE\x10\x02\x12\x18\n\x14JOB_STATUS_TOO_EARLY\x10\x03\x42%\n!com.qwak.ai.feature.store.reportsP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.feature_store.reports.report_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n!com.qwak.ai.feature.store.reportsP\001'
  _globals['_JOBSTATUS']._serialized_start=1014
  _globals['_JOBSTATUS']._serialized_end=1123
  _globals['_BATCHJOBINGESTIONDETAILS']._serialized_start=102
  _globals['_BATCHJOBINGESTIONDETAILS']._serialized_end=314
  _globals['_STREAMINGJOBINGESTIONDETAILS']._serialized_start=316
  _globals['_STREAMINGJOBINGESTIONDETAILS']._serialized_end=346
  _globals['_STREAMINGAGGREGATIONJOBINGESTIONDETAILS']._serialized_start=348
  _globals['_STREAMINGAGGREGATIONJOBINGESTIONDETAILS']._serialized_end=389
  _globals['_JOBREPORT']._serialized_start=392
  _globals['_JOBREPORT']._serialized_end=1012
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass, field
from pathlib import Path
from typing import List

from qwak.inner.build_logic.dependency_manager_type import DependencyManagerType
from qwak.inner.build_logic.interface.build_logger_interface import BuildLogger


@dataclass
class DependencyFileObject:
    dep_file_name: List[str]
    lock_file_name: str = field(default="")


DEPS_MANAGER_FILE_MAP = {
    DependencyManagerType.PIP: DependencyFileObject(dep_file_name=["requirements.txt"]),
    DependencyManagerType.POETRY: DependencyFileObject(
        dep_file_name=["pyproject.toml"], lock_file_name="poetry.lock"
    ),
    DependencyManagerType.CONDA: DependencyFileObject(
        dep_file_name=["conda.yml", "conda.yaml"]
    ),
}


def find_dependency_files(model_uri, main_dir, build_logger: BuildLogger):
    dependency_manager_type = None
    model_relative_dependency_file = None
    model_relative_dependency_lock_file = None
    for dep_type, dependency_file_object in DEPS_MANAGER_FILE_MAP.items():
        for filename in dependency_file_object.dep_file_name:
            dep_file_path = find_file_location(model_uri, main_dir, filename)
            if dep_file_path:
                build_logger.info(
                    f"Found dependency type: {dep_type.name} by file: {dep_file_path}"
                )
                dependency_manager_type = dep_type
                model_relative_dependency_file = dep_file_path.relative_to(model_uri)
                if dependency_file_object.lock_file_name:
                    dep_lock_file_path = find_file_location(
                        model_uri,
                        main_dir,
                        dependency_file_object.lock_file_name,
                    )
                    if dep_lock_file_path:
                        build_logger.info(
                            f"Found dependency lock file {dep_lock_file_path}"
                        )
                        model_relative_dependency_lock_file = (
                            dep_lock_file_path.relative_to(model_uri)
                        )
                break

    return (
        dependency_manager_type,
        model_relative_dependency_file,
        model_relative_dependency_lock_file,
    )


def find_file_location(model_uri, main_dir, filename) -> Path:
    file_locations: List[Path] = [
        model_uri / filename,
        model_uri / main_dir / filename,
    ]
    for file in file_locations:
        if file.is_file():
            return file



from _qwak_proto.qwak.feature_store.entities.entity_pb2 import EntitySpec
from _qwak_proto.qwak.feature_store.entities.entity_service_pb2 import (
    CreateEntityRequest,
    CreateEntityResponse,
    DeleteEntityRequest,
    DeleteEntityResponse,
    GetEntityByNameRequest,
    GetEntityByNameResponse,
    ListEntitiesRequest,
    ListEntitiesResponse,
    UpdateEntityRequest,
    UpdateEntityResponse,
)
from _qwak_proto.qwak.feature_store.entities.entity_service_pb2_grpc import (
    EntityServiceStub,
)
from _qwak_proto.qwak.feature_store.features.feature_set_pb2 import FeatureSetSpec
from _qwak_proto.qwak.feature_store.features.feature_set_service_pb2 import (
    CreateUploadURLRequestV1,
    DeleteFeatureSetRequest,
    DeleteFeatureSetResponse,
    DeleteFeaturesetVersionRequest,
    GetEnvToFeatureSetsMappingRequest,
    GetEnvToFeatureSetsMappingResponse,
    GetFeatureSetByNameRequest,
    GetFeatureSetByNameResponse,
    GetFeaturesetSchedulingStateRequest,
    GetFeaturesetSchedulingStateResponse,
    ListFeatureSetsRequest,
    ListFeatureSetsResponse,
    ListFeaturesetVersionsByNameRequest,
    ListFeaturesetVersionsByNameResponse,
    PauseFeaturesetRequest,
    RegisterFeatureSetRequest,
    RegisterFeatureSetResponse,
    ResumeFeaturesetRequest,
    RunBatchFeatureSetRequest,
    RunBatchFeatureSetResponse,
    RunParquetBatchFeatureSetRequest,
    SetActiveFeaturesetVersionRequest,
    UpdateFeatureSetRequest,
    UpdateFeatureSetResponse,
)
from _qwak_proto.qwak.feature_store.features.feature_set_service_pb2_grpc import (
    FeatureSetServiceStub,
)
from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    Transformation as ProtoTransformation,
)
from _qwak_proto.qwak.feature_store.sources.data_source_service_pb2 import (
    CreateDataSourceRequest,
    CreateDataSourceResponse,
    CreateDataSourceUploadURLRequest,
    DeleteDataSourceRequest,
    DeleteDataSourceResponse,
    GetDataSourceByNameRequest,
    GetDataSourceByNameResponse,
    ListDataSourcesRequest,
    ListDataSourcesResponse,
    UpdateDataSourceRequest,
    UpdateDataSourceResponse,
)
from _qwak_proto.qwak.feature_store.sources.data_source_service_pb2_grpc import (
    DataSourceServiceStub,
)
from _qwak_proto.qwak.feature_store.v1.internal.data_source.data_source_service_pb2 import (
    GetDataSourceSourceCodeUploadRequest,
    GetDataSourceSourceCodeUploadResponse,
)
from _qwak_proto.qwak.feature_store.v1.internal.data_source.data_source_service_pb2_grpc import (
    DataSourceServiceStub as InternalDataSourceServiceStub,
)
from dependency_injector.wiring import Provide
from grpc import RpcError, StatusCode
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer
from qwak.inner.tool.grpc.grpc_try_wrapping import grpc_try_catch_wrapper


class FeatureRegistryClient:
    """
    Used for interacting with Feature Registry endpoints
    """

    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._features_service = FeatureSetServiceStub(grpc_channel)
        self._entity_service = EntityServiceStub(grpc_channel)
        self._sources_service = DataSourceServiceStub(grpc_channel)
        self._internal_data_sources_service = InternalDataSourceServiceStub(
            grpc_channel
        )

    @grpc_try_catch_wrapper(
        "Failed to get a presigned url", reraise_non_grpc_error_original_exception=True
    )
    def get_datasource_source_code_presign_url(
        self, ds_name: str, repository_name: str
    ) -> GetDataSourceSourceCodeUploadResponse:
        return self._internal_data_sources_service.GetDataSourceSourceCodeUploadURL(
            GetDataSourceSourceCodeUploadRequest(
                data_source_name=ds_name, repository_name=repository_name
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to list features", reraise_non_grpc_error_original_exception=True
    )
    def list_feature_sets(self) -> ListFeatureSetsResponse:
        """
        Returns:
            ListFeatureSetsResponse
        """
        return self._features_service.ListFeatureSets(ListFeatureSetsRequest())

    def get_entity_by_name(self, entity_name: str) -> GetEntityByNameResponse:
        """
        Args:
            entity_name: entity name to get
        Returns:
            GetEntityByNameResponse
        """
        return self._fetch_by_name(
            lambda name: self._entity_service.GetEntityByName(
                GetEntityByNameRequest(entity_name=name)
            ),
            entity_name,
            "entity",
        )

    @grpc_try_catch_wrapper(
        "Failed to delete feature", reraise_non_grpc_error_original_exception=True
    )
    def delete_feature_set(self, feature_set_id: str) -> DeleteFeatureSetResponse:
        """
        Args:
            feature_set_id: feature set id to delete
        Returns:
            DeleteFeatureSetResponse
        """
        return self._features_service.DeleteFeatureSet(
            DeleteFeatureSetRequest(feature_set_id=feature_set_id)
        )

    @grpc_try_catch_wrapper(
        "Failed to delete entity", reraise_non_grpc_error_original_exception=True
    )
    def delete_entity(self, entity_id: str) -> DeleteEntityResponse:
        """
        Args:
            entity_id: entity id to delete
        Returns:
            DeleteEntityResponse
        """
        return self._entity_service.DeleteEntity(
            DeleteEntityRequest(entity_id=entity_id)
        )

    @grpc_try_catch_wrapper(
        "Failed to delete data source", reraise_non_grpc_error_original_exception=True
    )
    def delete_data_source(self, data_source_id: str) -> DeleteDataSourceResponse:
        """
        Args:
            data_source_id: data source id to delete
        Returns:
            DeleteDataSourceResponse
        """
        return self._sources_service.DeleteDataSource(
            DeleteDataSourceRequest(data_source_id=data_source_id)
        )

    @grpc_try_catch_wrapper(
        "Failed to run feature", reraise_non_grpc_error_original_exception=True
    )
    def run_feature_set(self, feature_set_name: str) -> RunBatchFeatureSetResponse:
        """
        Trigger a single ETL run for that feature set name
        Args:
            feature_set_name: feature set name to run
        Returns:
            :return RunBatchFeatureSetResponse with the execution id
        """
        return self._features_service.RunBatchFeatureSet(
            RunBatchFeatureSetRequest(feature_set_name=feature_set_name)
        )

    @grpc_try_catch_wrapper(
        "Failed to run feature", reraise_non_grpc_error_original_exception=True
    )
    def run_parquet_batch_feature_set(
        self, feature_set_name: str, parquet_location: str, timestamp_column: str
    ):
        """
        Trigger a single ETL run on an explicit parquet source for that feature set name
        Args:
            feature_set_name: feature set name to run
            parquet_location: parquet location to read from
            timestamp_column: the name of the timestamp column
        Returns:
            None
        """
        self._features_service.RunParquetBatchFeatureSet(
            RunParquetBatchFeatureSetRequest(
                feature_set_name=feature_set_name,
                parquet_location=parquet_location,
                timestamp_column=timestamp_column,
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to update entity", reraise_non_grpc_error_original_exception=True
    )
    def update_entity(
        self, entity_id: str, proto_entity_spec: EntitySpec
    ) -> UpdateEntityResponse:
        """
        Args:
            entity_id: entity id to update
            proto_entity_spec: entity spec proto to update
        Returns:
            UpdateEntityResponse
        """
        return self._entity_service.UpdateEntity(
            UpdateEntityRequest(entity_id=entity_id, entity_spec=proto_entity_spec)
        )

    @grpc_try_catch_wrapper(
        "Failed to create entity", reraise_non_grpc_error_original_exception=True
    )
    def create_entity(self, proto_entity_spec: EntitySpec) -> CreateEntityResponse:
        """
        Args:
            proto_entity_spec: entity spec proto to create
        Returns:
            CreateEntityResponse
        """
        return self._entity_service.CreateEntity(
            CreateEntityRequest(entity_spec=proto_entity_spec)
        )

    @grpc_try_catch_wrapper(
        "Failed to list entities", reraise_non_grpc_error_original_exception=True
    )
    def list_entities(self) -> ListEntitiesResponse:
        """
        Returns:
             ListEntitiesResponse
        """
        return self._entity_service.ListEntities(ListEntitiesRequest())

    @grpc_try_catch_wrapper(
        "Failed to list data sources", reraise_non_grpc_error_original_exception=True
    )
    def list_data_sources(self) -> ListDataSourcesResponse:
        """
        Returns:
            ListDataSourcesResponse
        """
        return self._sources_service.ListDataSources(ListDataSourcesRequest())

    def get_data_source_by_name(
        self, data_source_name: str
    ) -> GetDataSourceByNameResponse:
        """
        Args:
            data_source_name: data source name to get
        Returns:
            GetDataSourceByNameResponse
        """
        return self._fetch_by_name(
            lambda name: self._sources_service.GetDataSourceByName(
                GetDataSourceByNameRequest(data_source_name=name)
            ),
            data_source_name,
            "data source",
        )

    @grpc_try_catch_wrapper(
        "Failed to update data source", reraise_non_grpc_error_original_exception=True
    )
    def update_data_source(
        self,
        data_source_id,
        proto_data_source: FeatureSetSpec,
    ) -> UpdateDataSourceResponse:
        """
        Args:
            data_source_id: data source id to update
            proto_data_source: proto data source spec to update
        Returns:
             UpdateDataSourceResponse
        """
        return self._sources_service.UpdateDataSource(
            UpdateDataSourceRequest(
                data_source_id=data_source_id,
                data_source_spec=proto_data_source,
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to create data source", reraise_non_grpc_error_original_exception=True
    )
    def create_data_source(
        self, proto_data_source: FeatureSetSpec
    ) -> CreateDataSourceResponse:
        """
        Args:
            proto_data_source: proto data source spec to create
        Returns:
            CreateDataSourceResponse
        """
        return self._sources_service.CreateDataSource(
            CreateDataSourceRequest(data_source_spec=proto_data_source)
        )

    def get_feature_set_by_name(self, feature_set_name) -> GetFeatureSetByNameResponse:
        """
        Args:
            feature_set_name: feature set name to get
        Returns:
             GetFeatureSetByNameResponse
        """
        return self._fetch_by_name(
            lambda name: self._features_service.GetFeatureSetByName(
                GetFeatureSetByNameRequest(feature_set_name=name)
            ),
            feature_set_name,
            "feature set",
        )

    @grpc_try_catch_wrapper(
        "Failed to update feature set", reraise_non_grpc_error_original_exception=True
    )
    def update_feature_set(
        self,
        feature_set_id: str,
        proto_feature_set: FeatureSetSpec,
    ) -> UpdateFeatureSetResponse:
        """
        Args:
            feature_set_id: feature set id to update
            proto_feature_set: features set to proto serialization
        Returns:
            UpdateFeatureSetResponse
        """
        return self._features_service.UpdateFeatureSet(
            UpdateFeatureSetRequest(
                feature_set_id=feature_set_id, feature_set_spec=proto_feature_set
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to create feature set", reraise_non_grpc_error_original_exception=True
    )
    def create_feature_set(
        self, proto_feature_set: FeatureSetSpec
    ) -> RegisterFeatureSetResponse:
        """
        Args:
            proto_feature_set: features set to proto serialization
        Returns:
            RegisterFeatureSetResponse
        """
        return self._features_service.RegisterFeatureSet(
            RegisterFeatureSetRequest(feature_set_spec=proto_feature_set)
        )

    @staticmethod
    def _fetch_by_name(extract_function, name, fs_object_type):
        """
        Args:
            extract_function: function to run on name param
            name: name of object to fetch
            fs_object_type: fs object type for exception reporting
        Returns:
            extract_function output
        """
        try:
            return extract_function(name)
        except RpcError as e:
            if e.code() == StatusCode.NOT_FOUND:
                return None
            else:
                raise QwakException(
                    f"Failed to fetch {fs_object_type} by name [{name}], error code: [{e.details()}]"
                )

    def get_presigned_url_v1(
        self, feature_set_name: str, transformation: ProtoTransformation
    ) -> str:
        """
        Get presigned url according feature set name and transformation type
        @param feature_set_name: featureset name for which we're getting the presigned URL
        @type feature_set_name: str
        @param transformation: The proto object of the transformation for which we're getting the presigned URL
        @type transformation: ProtoTransformation
        @return:
        @rtype:
        """
        try:
            return self._features_service.CreateUploadURLV1(
                CreateUploadURLRequestV1(
                    featureset_name=feature_set_name,
                    transformation=transformation,
                )
            ).upload_url
        except RpcError as e:
            raise QwakException(
                f"Failed to get presigned url for featureset name {feature_set_name} and transformation type "
                f"{type(transformation)}, error code: [{e.details()}]"
            )

    @grpc_try_catch_wrapper(
        "Failed to get presigned url for data source: {data_source_name}, object name: {object_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def get_data_source_presigned_url_v1(
        self, data_source_name: str, object_name: str
    ) -> str:
        """
        create a pre-sign url for data source according to qwak path convention that includes a timestamp millis
        to avoid using the same path. This method is meant to be used by the new sdk, which passes the path along
        in the proto message.
        Args:
            data_source_name: str:  data source name
            object_name: str object name
        Returns:
            presigned url: str
        """
        return self._sources_service.CreateDataSourceUploadURLV1(
            CreateDataSourceUploadURLRequest(
                data_source_name=data_source_name, object_name=object_name
            )
        ).upload_url

    @grpc_try_catch_wrapper(
        "Failed to query scheduling state for Featureset {feature_set_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def get_feature_set_scheduling_state(
        self, feature_set_name: str
    ) -> GetFeaturesetSchedulingStateResponse:
        return self._features_service.GetFeaturesetSchedulingState(
            GetFeaturesetSchedulingStateRequest(featureset_name=feature_set_name)
        )

    @grpc_try_catch_wrapper(
        "Failed to pause Featureset {feature_set_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def pause_feature_set(self, feature_set_name: str):
        self._features_service.PauseFeatureset(
            PauseFeaturesetRequest(featureset_name=feature_set_name)
        )

    @grpc_try_catch_wrapper(
        "Failed to resume Featureset {feature_set_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def resume_feature_set(self, feature_set_name: str):
        self._features_service.ResumeFeatureset(
            ResumeFeaturesetRequest(featureset_name=feature_set_name)
        )

    @grpc_try_catch_wrapper(
        "Failed list Featureset versions for Featureset: {featureset_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def list_featureset_versions(
        self, featureset_name: str
    ) -> ListFeaturesetVersionsByNameResponse:
        return self._features_service.ListFeaturesetVersionsByName(
            ListFeaturesetVersionsByNameRequest(featureset_name=featureset_name)
        )

    @grpc_try_catch_wrapper(
        "Failed to set active version: {version_number} for Featureset: {featureset_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def set_active_featureset_version(self, featureset_name: str, version_number: int):
        self._features_service.SetActiveFeaturesetVersion(
            SetActiveFeaturesetVersionRequest(
                featureset_name=featureset_name,
                featureset_version_number=version_number,
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to delete version: {version_number} for Featureset: {featureset_name}",
        reraise_non_grpc_error_original_exception=True,
    )
    def delete_featureset_version(self, featureset_name: str, version_number: int):
        self._features_service.DeleteFeaturesetVersion(
            DeleteFeaturesetVersionRequest(
                featureset_name=featureset_name,
                featureset_version_number=version_number,
            )
        )

    @grpc_try_catch_wrapper(
        "Failed to get env to featuresets mapping",
        reraise_non_grpc_error_original_exception=True,
    )
    def get_env_to_featuresets_mapping(self) -> GetEnvToFeatureSetsMappingResponse:
        return self._features_service.GetEnvToFeatureSetsMapping(
            GetEnvToFeatureSetsMappingRequest()
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/fitness_service/fitness.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.fitness_service import constructs_pb2 as qwak_dot_fitness__service_dot_constructs__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"qwak/fitness_service/fitness.proto\x12\x14qwak.fitness.service\x1a%qwak/fitness_service/constructs.proto\"\xca\x04\n\tBuildSpec\x12?\n\x10\x62uild_properties\x18\x01 \x01(\x0b\x32%.qwak.fitness.service.BuildProperties\x12\x31\n\tbuild_env\x18\x02 \x01(\x0b\x32\x1e.qwak.fitness.service.BuildEnv\x12;\n\rcpu_resources\x18\x03 \x01(\x0b\x32\".qwak.fitness.service.CpuResourcesH\x00\x12;\n\rgpu_resources\x18\x07 \x01(\x0b\x32\".qwak.fitness.service.GpuResourcesH\x00\x12\x0f\n\x07verbose\x18\x04 \x01(\x05\x12\x14\n\x0c\x62uild_config\x18\x05 \x01(\t\x12\x15\n\rbuild_v1_flag\x18\x06 \x01(\x08\x12\x44\n\x13\x62uild_properties_v1\x18\x08 \x01(\x0b\x32\'.qwak.fitness.service.BuildPropertiesV1\x12=\n\x0fpurchase_option\x18\t \x01(\x0e\x32$.qwak.fitness.service.PurchaseOption\x12\x1c\n\x14\x62uild_destined_image\x18\n \x01(\t\x12\"\n\x1aprovision_instance_timeout\x18\x0b \x01(\x05\x12\x1f\n\x17\x61rtifactory_project_key\x18\x0c \x01(\t\x12\x1c\n\x14\x61rtifactory_base_url\x18\r \x01(\tB\x0b\n\tResources\"g\n\x11\x42uildPropertiesV1\x12\x18\n\x10\x62uild_config_url\x18\x01 \x01(\t\x12\x1a\n\x12qwak_sdk_wheel_url\x18\x02 \x01(\t\x12\x1c\n\x14qwak_sdk_version_url\x18\x03 \x01(\t*e\n\x0ePurchaseOption\x12\x1b\n\x17INVALID_PURCHASE_OPTION\x10\x00\x12\x18\n\x14SPOT_PURCHASE_OPTION\x10\x01\x12\x1c\n\x18ONDEMAND_PURCHASE_OPTION\x10\x02\x42#\n\x1f\x63om.qwak.ai.fitness.service.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.fitness_service.fitness_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.fitness.service.apiP\001'
  _globals['_PURCHASEOPTION']._serialized_start=793
  _globals['_PURCHASEOPTION']._serialized_end=894
  _globals['_BUILDSPEC']._serialized_start=100
  _globals['_BUILDSPEC']._serialized_end=686
  _globals['_BUILDPROPERTIESV1']._serialized_start=688
  _globals['_BUILDPROPERTIESV1']._serialized_end=791
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/runtime_configuration/v0/external/databricks/auth.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.administration.runtime_configuration.v0.creds import secret_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_creds_dot_secret__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nKqwak/administration/runtime_configuration/v0/external/databricks/auth.proto\x12@qwak.administration.runtime_configuration.v0.external.databricks\x1a?qwak/administration/runtime_configuration/v0/creds/secret.proto\"\x9f\x02\n\x0e\x44\x61tabricksAuth\x12r\n\x0f\x64\x62_access_token\x18\x01 \x01(\x0b\x32W.qwak.administration.runtime_configuration.v0.external.databricks.DatabricksAccessTokenH\x00\x12\x8b\x01\n\x18\x64\x62_principal_credentials\x18\x02 \x01(\x0b\x32g.qwak.administration.runtime_configuration.v0.external.databricks.DatabricksServicePrincipalCredentialsH\x00\x42\x0b\n\tauth_type\"\xb6\x01\n%DatabricksServicePrincipalCredentials\x12\x11\n\tclient_id\x18\x01 \x01(\t\x12`\n\rclient_secret\x18\x02 \x01(\x0b\x32G.qwak.administration.runtime_configuration.v0.creds.secret.SecretKeyRefH\x00\x42\x18\n\x16\x63lient_secret_location\"~\n\x15\x44\x61tabricksAccessToken\x12Y\n\x06secret\x18\x01 \x01(\x0b\x32G.qwak.administration.runtime_configuration.v0.creds.secret.SecretKeyRefH\x00\x42\n\n\x08locationB\xef\x01\n9com.jfrog.ml.runtime_configuration.v0.external.databricksP\x01Z\xaf\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0/external/databricks;external_databricksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.administration.runtime_configuration.v0.external.databricks.auth_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n9com.jfrog.ml.runtime_configuration.v0.external.databricksP\001Z\257\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0/external/databricks;external_databricks'
  _globals['_DATABRICKSAUTH']._serialized_start=211
  _globals['_DATABRICKSAUTH']._serialized_end=498
  _globals['_DATABRICKSSERVICEPRINCIPALCREDENTIALS']._serialized_start=501
  _globals['_DATABRICKSSERVICEPRINCIPALCREDENTIALS']._serialized_end=683
  _globals['_DATABRICKSACCESSTOKEN']._serialized_start=685
  _globals['_DATABRICKSACCESSTOKEN']._serialized_end=811
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _MemoryUnitApi:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MemoryUnitApiEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MemoryUnitApi.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN_MEMORY_UNIT: _MemoryUnitApi.ValueType  # 0
    MIB: _MemoryUnitApi.ValueType  # 1
    GIB: _MemoryUnitApi.ValueType  # 2

class MemoryUnitApi(_MemoryUnitApi, metaclass=_MemoryUnitApiEnumTypeWrapper): ...

UNKNOWN_MEMORY_UNIT: MemoryUnitApi.ValueType  # 0
MIB: MemoryUnitApi.ValueType  # 1
GIB: MemoryUnitApi.ValueType  # 2
global___MemoryUnitApi = MemoryUnitApi

class _InputFileType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _InputFileTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_InputFileType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNDEFINED_INPUT_FILE_TYPE: _InputFileType.ValueType  # 0
    CSV_INPUT_FILE_TYPE: _InputFileType.ValueType  # 1
    FEATHER_INPUT_FILE_TYPE: _InputFileType.ValueType  # 2
    PARQUET_INPUT_FILE_TYPE: _InputFileType.ValueType  # 3

class InputFileType(_InputFileType, metaclass=_InputFileTypeEnumTypeWrapper): ...

UNDEFINED_INPUT_FILE_TYPE: InputFileType.ValueType  # 0
CSV_INPUT_FILE_TYPE: InputFileType.ValueType  # 1
FEATHER_INPUT_FILE_TYPE: InputFileType.ValueType  # 2
PARQUET_INPUT_FILE_TYPE: InputFileType.ValueType  # 3
global___InputFileType = InputFileType

class _OutputFileType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _OutputFileTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_OutputFileType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNDEFINED_OUTPUT_FILE_TYPE: _OutputFileType.ValueType  # 0
    CSV_OUTPUT_FILE_TYPE: _OutputFileType.ValueType  # 1
    FEATHER_OUTPUT_FILE_TYPE: _OutputFileType.ValueType  # 2
    PARQUET_OUTPUT_FILE_TYPE: _OutputFileType.ValueType  # 3

class OutputFileType(_OutputFileType, metaclass=_OutputFileTypeEnumTypeWrapper): ...

UNDEFINED_OUTPUT_FILE_TYPE: OutputFileType.ValueType  # 0
CSV_OUTPUT_FILE_TYPE: OutputFileType.ValueType  # 1
FEATHER_OUTPUT_FILE_TYPE: OutputFileType.ValueType  # 2
PARQUET_OUTPUT_FILE_TYPE: OutputFileType.ValueType  # 3
global___OutputFileType = OutputFileType

class _GpuType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _GpuTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_GpuType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_GPU: _GpuType.ValueType  # 0
    NVIDIA_K80: _GpuType.ValueType  # 1
    NVIDIA_V100: _GpuType.ValueType  # 2
    NVIDIA_A100: _GpuType.ValueType  # 3
    NVIDIA_T4: _GpuType.ValueType  # 4
    NVIDIA_A10G: _GpuType.ValueType  # 5
    NVIDIA_L4: _GpuType.ValueType  # 6
    NVIDIA_T4_1_4_15: _GpuType.ValueType  # 7
    """More specific node types
    t4.xl
    """
    NVIDIA_T4_1_8_30: _GpuType.ValueType  # 8
    """t4.2xl"""
    NVIDIA_T4_1_16_60: _GpuType.ValueType  # 9
    """t4.4xl"""
    NVIDIA_A100_80GB_8_96_1360: _GpuType.ValueType  # 10
    """a100.8xl"""
    NVIDIA_V100_1_8_52: _GpuType.ValueType  # 11
    """a100.xl"""
    NVIDIA_V100_4_32_208: _GpuType.ValueType  # 12
    """v100.4xl"""

class GpuType(_GpuType, metaclass=_GpuTypeEnumTypeWrapper): ...

INVALID_GPU: GpuType.ValueType  # 0
NVIDIA_K80: GpuType.ValueType  # 1
NVIDIA_V100: GpuType.ValueType  # 2
NVIDIA_A100: GpuType.ValueType  # 3
NVIDIA_T4: GpuType.ValueType  # 4
NVIDIA_A10G: GpuType.ValueType  # 5
NVIDIA_L4: GpuType.ValueType  # 6
NVIDIA_T4_1_4_15: GpuType.ValueType  # 7
"""More specific node types
t4.xl
"""
NVIDIA_T4_1_8_30: GpuType.ValueType  # 8
"""t4.2xl"""
NVIDIA_T4_1_16_60: GpuType.ValueType  # 9
"""t4.4xl"""
NVIDIA_A100_80GB_8_96_1360: GpuType.ValueType  # 10
"""a100.8xl"""
NVIDIA_V100_1_8_52: GpuType.ValueType  # 11
"""a100.xl"""
NVIDIA_V100_4_32_208: GpuType.ValueType  # 12
"""v100.4xl"""
global___GpuType = GpuType

class _CloudProvider:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CloudProviderEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CloudProvider.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN_CLOUD_PROVIDER: _CloudProvider.ValueType  # 0
    AWS: _CloudProvider.ValueType  # 1
    GCP: _CloudProvider.ValueType  # 2

class CloudProvider(_CloudProvider, metaclass=_CloudProviderEnumTypeWrapper): ...

UNKNOWN_CLOUD_PROVIDER: CloudProvider.ValueType  # 0
AWS: CloudProvider.ValueType  # 1
GCP: CloudProvider.ValueType  # 2
global___CloudProvider = CloudProvider

class ListInferenceJobFilesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUCKET_FIELD_NUMBER: builtins.int
    DIRECTORY_PATH_FIELD_NUMBER: builtins.int
    TOKEN_SECRET_FIELD_NUMBER: builtins.int
    SECRET_SECRET_FIELD_NUMBER: builtins.int
    SECRET_SERVICE_URL_FIELD_NUMBER: builtins.int
    ROLE_ARN_FIELD_NUMBER: builtins.int
    AWS_CREDENTIALS_FIELD_NUMBER: builtins.int
    GCP_CREDENTIALS_FIELD_NUMBER: builtins.int
    GROUPED_TASK_DETAILS_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    bucket: builtins.str
    """The bucket in cloud storage"""
    directory_path: builtins.str
    """The path in the bucket to read the files from"""
    token_secret: builtins.str
    """The qwak secret the bucket's token is stored in"""
    secret_secret: builtins.str
    """The qwak secret the bucket's secret is stored in"""
    secret_service_url: builtins.str
    """The qwak secret service url for that environment"""
    role_arn: builtins.str
    """The role arn to assume"""
    @property
    def aws_credentials(self) -> global___AwsCredentials: ...
    @property
    def gcp_credentials(self) -> global___GcpCredentials: ...
    @property
    def grouped_task_details(self) -> global___GroupedTaskDetails: ...
    environment_id: builtins.str
    def __init__(
        self,
        *,
        bucket: builtins.str = ...,
        directory_path: builtins.str = ...,
        token_secret: builtins.str = ...,
        secret_secret: builtins.str = ...,
        secret_service_url: builtins.str = ...,
        role_arn: builtins.str = ...,
        aws_credentials: global___AwsCredentials | None = ...,
        gcp_credentials: global___GcpCredentials | None = ...,
        grouped_task_details: global___GroupedTaskDetails | None = ...,
        environment_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aws_credentials", b"aws_credentials", "cloud_client_credentials", b"cloud_client_credentials", "gcp_credentials", b"gcp_credentials", "grouped_task_details", b"grouped_task_details", "task_flow", b"task_flow"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aws_credentials", b"aws_credentials", "bucket", b"bucket", "cloud_client_credentials", b"cloud_client_credentials", "directory_path", b"directory_path", "environment_id", b"environment_id", "gcp_credentials", b"gcp_credentials", "grouped_task_details", b"grouped_task_details", "role_arn", b"role_arn", "secret_secret", b"secret_secret", "secret_service_url", b"secret_service_url", "task_flow", b"task_flow", "token_secret", b"token_secret"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["cloud_client_credentials", b"cloud_client_credentials"]) -> typing_extensions.Literal["aws_credentials", "gcp_credentials"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["task_flow", b"task_flow"]) -> typing_extensions.Literal["grouped_task_details"] | None: ...

global___ListInferenceJobFilesRequest = ListInferenceJobFilesRequest

class GroupedTaskDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONCURRENTTASKS_FIELD_NUMBER: builtins.int
    DESTINATION_PATH_FIELD_NUMBER: builtins.int
    DESTINATION_BUCKET_FIELD_NUMBER: builtins.int
    concurrentTasks: builtins.int
    """concurrent task to divide files to group"""
    destination_path: builtins.str
    """path to upload temp point files"""
    destination_bucket: builtins.str
    """The destination bucket in cloud storage"""
    def __init__(
        self,
        *,
        concurrentTasks: builtins.int = ...,
        destination_path: builtins.str = ...,
        destination_bucket: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["concurrentTasks", b"concurrentTasks", "destination_bucket", b"destination_bucket", "destination_path", b"destination_path"]) -> None: ...

global___GroupedTaskDetails = GroupedTaskDetails

class CleanStorageObjectFromCloudStorageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUCKET_FIELD_NUMBER: builtins.int
    OBJECT_PREFIX_FIELD_NUMBER: builtins.int
    SECRET_SERVICE_URL_FIELD_NUMBER: builtins.int
    AWS_CREDENTIALS_FIELD_NUMBER: builtins.int
    GCP_CREDENTIALS_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    bucket: builtins.str
    """The bucket in cloud storage"""
    @property
    def object_prefix(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The path in the objects to delete"""
    secret_service_url: builtins.str
    """The qwak secret service url for that environment"""
    @property
    def aws_credentials(self) -> global___AwsCredentials: ...
    @property
    def gcp_credentials(self) -> global___GcpCredentials: ...
    environment_id: builtins.str
    def __init__(
        self,
        *,
        bucket: builtins.str = ...,
        object_prefix: collections.abc.Iterable[builtins.str] | None = ...,
        secret_service_url: builtins.str = ...,
        aws_credentials: global___AwsCredentials | None = ...,
        gcp_credentials: global___GcpCredentials | None = ...,
        environment_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aws_credentials", b"aws_credentials", "cloud_client_credentials", b"cloud_client_credentials", "gcp_credentials", b"gcp_credentials"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aws_credentials", b"aws_credentials", "bucket", b"bucket", "cloud_client_credentials", b"cloud_client_credentials", "environment_id", b"environment_id", "gcp_credentials", b"gcp_credentials", "object_prefix", b"object_prefix", "secret_service_url", b"secret_service_url"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["cloud_client_credentials", b"cloud_client_credentials"]) -> typing_extensions.Literal["aws_credentials", "gcp_credentials"] | None: ...

global___CleanStorageObjectFromCloudStorageRequest = CleanStorageObjectFromCloudStorageRequest

class AwsCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROLE_ARN_FIELD_NUMBER: builtins.int
    TOKEN_SECRET_FIELD_NUMBER: builtins.int
    SECRET_SECRET_FIELD_NUMBER: builtins.int
    role_arn: builtins.str
    """The role arn to assume"""
    token_secret: builtins.str
    """The qwak secret the bucket's token is stored in"""
    secret_secret: builtins.str
    """The qwak secret the bucket's secret is stored in"""
    def __init__(
        self,
        *,
        role_arn: builtins.str = ...,
        token_secret: builtins.str = ...,
        secret_secret: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["role_arn", b"role_arn", "secret_secret", b"secret_secret", "token_secret", b"token_secret"]) -> None: ...

global___AwsCredentials = AwsCredentials

class GcpCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVICE_ACCOUNT_JSON_KEY_SECRET_FIELD_NUMBER: builtins.int
    service_account_json_key_secret: builtins.str
    def __init__(
        self,
        *,
        service_account_json_key_secret: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["service_account_json_key_secret", b"service_account_json_key_secret"]) -> None: ...

global___GcpCredentials = GcpCredentials

class ListInferenceJobFilesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILE_NAMES_FIELD_NUMBER: builtins.int
    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    LIST_FILE_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def file_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The list of files in the folder"""
    success: builtins.bool
    """Whether the request was successful or not"""
    failure_reason: builtins.str
    """The failure description"""
    @property
    def list_file_details(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ListFileDetails]:
        """the list of file details"""
    def __init__(
        self,
        *,
        file_names: collections.abc.Iterable[builtins.str] | None = ...,
        success: builtins.bool = ...,
        failure_reason: builtins.str = ...,
        list_file_details: collections.abc.Iterable[global___ListFileDetails] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "file_names", b"file_names", "list_file_details", b"list_file_details", "success", b"success"]) -> None: ...

global___ListInferenceJobFilesResponse = ListInferenceJobFilesResponse

class ListFileDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IS_POINT_FILE_FIELD_NUMBER: builtins.int
    POINT_FILE_PATH_FIELD_NUMBER: builtins.int
    OBJECT_PATHS_FIELD_NUMBER: builtins.int
    is_point_file: builtins.bool
    """Is point file is used - for admiral flow (intermediate state till we removed kube flow - MLOPS-2201)"""
    point_file_path: builtins.str
    """point file path"""
    @property
    def object_paths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of inference files"""
    def __init__(
        self,
        *,
        is_point_file: builtins.bool = ...,
        point_file_path: builtins.str = ...,
        object_paths: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["is_point_file", b"is_point_file", "object_paths", b"object_paths", "point_file_path", b"point_file_path"]) -> None: ...

global___ListFileDetails = ListFileDetails

class CreateInferenceTaskExecutorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TASK_EXECUTOR_CONFIGURATION_FIELD_NUMBER: builtins.int
    INFERENCE_TASK_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def task_executor_configuration(self) -> global___TaskExecutorConfigurationMessage: ...
    @property
    def inference_task_configuration(self) -> global___InferenceTaskConfigurationMessage: ...
    def __init__(
        self,
        *,
        task_executor_configuration: global___TaskExecutorConfigurationMessage | None = ...,
        inference_task_configuration: global___InferenceTaskConfigurationMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inference_task_configuration", b"inference_task_configuration", "task_executor_configuration", b"task_executor_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inference_task_configuration", b"inference_task_configuration", "task_executor_configuration", b"task_executor_configuration"]) -> None: ...

global___CreateInferenceTaskExecutorRequest = CreateInferenceTaskExecutorRequest

class PrepareInferenceJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFERENCE_JOB_CONFIGURATION_MESSAGE_FIELD_NUMBER: builtins.int
    @property
    def inference_job_configuration_message(self) -> global___InferenceJobConfigurationMessage: ...
    def __init__(
        self,
        *,
        inference_job_configuration_message: global___InferenceJobConfigurationMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inference_job_configuration_message", b"inference_job_configuration_message"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inference_job_configuration_message", b"inference_job_configuration_message"]) -> None: ...

global___PrepareInferenceJobRequest = PrepareInferenceJobRequest

class PrepareInferenceJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    success: builtins.bool
    """Whether the request was successful or not"""
    failure_reason: builtins.str
    """The failure description"""
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
        failure_reason: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "success", b"success"]) -> None: ...

global___PrepareInferenceJobResponse = PrepareInferenceJobResponse

class TaskExecutorConfigurationMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFERENCE_JOB_ID_FIELD_NUMBER: builtins.int
    INFERENCE_TASK_ID_FIELD_NUMBER: builtins.int
    MODEL_IDENTIFIER_FIELD_NUMBER: builtins.int
    IMAGE_URL_FIELD_NUMBER: builtins.int
    BACKOFF_LIMIT_FIELD_NUMBER: builtins.int
    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    CUSTOM_IAM_ROLE_ARN_FIELD_NUMBER: builtins.int
    JOB_SIZE_FIELD_NUMBER: builtins.int
    PURCHASE_OPTION_FIELD_NUMBER: builtins.int
    IMAGE_PULL_SECRET_FIELD_NUMBER: builtins.int
    CUSTOM_SERVICE_ACCOUNT_SECRET_FIELD_NUMBER: builtins.int
    JFROG_TOKEN_API_SECRET_NAME_FIELD_NUMBER: builtins.int
    JFROG_BASE_URL_FIELD_NUMBER: builtins.int
    inference_job_id: builtins.str
    """The id of the inference job"""
    inference_task_id: builtins.str
    """The id of the current inference task"""
    @property
    def model_identifier(self) -> global___ModelIdentifier:
        """Model details"""
    image_url: builtins.str
    """The image url to use"""
    backoff_limit: builtins.int
    """How many times the current task can fail"""
    cpu: builtins.float
    """Cpu usage"""
    memory_amount: builtins.int
    """Memory usage"""
    memory_units: global___MemoryUnitApi.ValueType
    """Units type of memory"""
    environment_id: builtins.str
    custom_iam_role_arn: builtins.str
    @property
    def job_size(self) -> global___BatchJobResources: ...
    purchase_option: builtins.str
    """Whether it is spot/ondemand (default - spot)"""
    image_pull_secret: builtins.str
    """If the image saved in out source repository (jfrog e.g.)  and image pull secret is needed"""
    custom_service_account_secret: builtins.str
    """The custom service account json key secret for gcp"""
    jfrog_token_api_secret_name: builtins.str
    """The Token secret name for jfrog token"""
    jfrog_base_url: builtins.str
    """The Jfrog base url"""
    def __init__(
        self,
        *,
        inference_job_id: builtins.str = ...,
        inference_task_id: builtins.str = ...,
        model_identifier: global___ModelIdentifier | None = ...,
        image_url: builtins.str = ...,
        backoff_limit: builtins.int = ...,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnitApi.ValueType = ...,
        environment_id: builtins.str = ...,
        custom_iam_role_arn: builtins.str = ...,
        job_size: global___BatchJobResources | None = ...,
        purchase_option: builtins.str = ...,
        image_pull_secret: builtins.str = ...,
        custom_service_account_secret: builtins.str = ...,
        jfrog_token_api_secret_name: builtins.str = ...,
        jfrog_base_url: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["job_size", b"job_size", "model_identifier", b"model_identifier"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["backoff_limit", b"backoff_limit", "cpu", b"cpu", "custom_iam_role_arn", b"custom_iam_role_arn", "custom_service_account_secret", b"custom_service_account_secret", "environment_id", b"environment_id", "image_pull_secret", b"image_pull_secret", "image_url", b"image_url", "inference_job_id", b"inference_job_id", "inference_task_id", b"inference_task_id", "jfrog_base_url", b"jfrog_base_url", "jfrog_token_api_secret_name", b"jfrog_token_api_secret_name", "job_size", b"job_size", "memory_amount", b"memory_amount", "memory_units", b"memory_units", "model_identifier", b"model_identifier", "purchase_option", b"purchase_option"]) -> None: ...

global___TaskExecutorConfigurationMessage = TaskExecutorConfigurationMessage

class InferenceJobConfigurationMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFERENCE_JOB_ID_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    CUSTOM_IAM_ROLE_ARN_FIELD_NUMBER: builtins.int
    inference_job_id: builtins.str
    """The id of the inference job"""
    environment_id: builtins.str
    """Environment id"""
    custom_iam_role_arn: builtins.str
    """Custom IAm role"""
    def __init__(
        self,
        *,
        inference_job_id: builtins.str = ...,
        environment_id: builtins.str = ...,
        custom_iam_role_arn: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["custom_iam_role_arn", b"custom_iam_role_arn", "environment_id", b"environment_id", "inference_job_id", b"inference_job_id"]) -> None: ...

global___InferenceJobConfigurationMessage = InferenceJobConfigurationMessage

class ModelIdentifier(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    BRANCH_ID_FIELD_NUMBER: builtins.int
    MODEL_UUID_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    build_id: builtins.str
    branch_id: builtins.str
    model_uuid: builtins.str
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        build_id: builtins.str = ...,
        branch_id: builtins.str = ...,
        model_uuid: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["branch_id", b"branch_id", "build_id", b"build_id", "model_id", b"model_id", "model_uuid", b"model_uuid"]) -> None: ...

global___ModelIdentifier = ModelIdentifier

class ListFilesPreStepConfiguration(google.protobuf.message.Message):
    """will be deprecated soon"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_PATH_FIELD_NUMBER: builtins.int
    LIST_FILES_FILE_NAME_FIELD_NUMBER: builtins.int
    TOTAL_NUMBER_OF_TASKS_FIELD_NUMBER: builtins.int
    TASK_INDEX_FIELD_NUMBER: builtins.int
    source_path: builtins.str
    """Directory path in source bucket of data files to run inference on"""
    list_files_file_name: builtins.str
    """Name for the file to save the list of files in the source path"""
    total_number_of_tasks: builtins.int
    """Total number of inference tasks, used to divide the files list into similarly sized groups"""
    @property
    def task_index(self) -> google.protobuf.wrappers_pb2.Int32Value:
        """Index of the specific task, used to determine on which files group to run inference on. value 0 is legit"""
    def __init__(
        self,
        *,
        source_path: builtins.str = ...,
        list_files_file_name: builtins.str = ...,
        total_number_of_tasks: builtins.int = ...,
        task_index: google.protobuf.wrappers_pb2.Int32Value | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["task_index", b"task_index"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["list_files_file_name", b"list_files_file_name", "source_path", b"source_path", "task_index", b"task_index", "total_number_of_tasks", b"total_number_of_tasks"]) -> None: ...

global___ListFilesPreStepConfiguration = ListFilesPreStepConfiguration

class InferenceTaskConfigurationMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_BUCKET_FIELD_NUMBER: builtins.int
    DESTINATION_BUCKET_FIELD_NUMBER: builtins.int
    FILEPATH_FIELD_NUMBER: builtins.int
    DESTINATION_PATH_FIELD_NUMBER: builtins.int
    INPUT_FILE_TYPE_FIELD_NUMBER: builtins.int
    OUTPUT_FILE_TYPE_FIELD_NUMBER: builtins.int
    TOKEN_SECRET_FIELD_NUMBER: builtins.int
    SECRET_SECRET_FIELD_NUMBER: builtins.int
    PARAMETERS_FIELD_NUMBER: builtins.int
    CLOUD_PROVIDER_FIELD_NUMBER: builtins.int
    SERVICE_ACCOUNT_JSON_KEY_SECRET_FIELD_NUMBER: builtins.int
    IS_POINT_FILE_PATH_FIELD_NUMBER: builtins.int
    LIST_FILES_PRE_STEP_CONFIGURATION_FIELD_NUMBER: builtins.int
    source_bucket: builtins.str
    """The source bucket in cloud storage"""
    destination_bucket: builtins.str
    """The destination bucket in cloud storage"""
    filepath: builtins.str
    """The full file path to process"""
    destination_path: builtins.str
    """The destination path to save results to"""
    input_file_type: global___InputFileType.ValueType
    """The input file type"""
    output_file_type: global___OutputFileType.ValueType
    """The output file type"""
    token_secret: builtins.str
    """The qwak secret the bucket's token is stored in"""
    secret_secret: builtins.str
    """The qwak secret the bucket's secret is stored in"""
    @property
    def parameters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BatchJobParameter]:
        """User provided parameters which will be passed to the batch execution job as environment variables"""
    cloud_provider: global___CloudProvider.ValueType
    """The cloud provider"""
    service_account_json_key_secret: builtins.str
    """The service account json key secret"""
    is_point_file_path: builtins.bool
    """Is the file contains points to inference files"""
    @property
    def list_files_pre_step_configuration(self) -> global___ListFilesPreStepConfiguration:
        """Optional pre step configuration, used to list files in the source directory and run inference on subset of files"""
    def __init__(
        self,
        *,
        source_bucket: builtins.str = ...,
        destination_bucket: builtins.str = ...,
        filepath: builtins.str = ...,
        destination_path: builtins.str = ...,
        input_file_type: global___InputFileType.ValueType = ...,
        output_file_type: global___OutputFileType.ValueType = ...,
        token_secret: builtins.str = ...,
        secret_secret: builtins.str = ...,
        parameters: collections.abc.Iterable[global___BatchJobParameter] | None = ...,
        cloud_provider: global___CloudProvider.ValueType = ...,
        service_account_json_key_secret: builtins.str = ...,
        is_point_file_path: builtins.bool = ...,
        list_files_pre_step_configuration: global___ListFilesPreStepConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["list_files_pre_step_configuration", b"list_files_pre_step_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cloud_provider", b"cloud_provider", "destination_bucket", b"destination_bucket", "destination_path", b"destination_path", "filepath", b"filepath", "input_file_type", b"input_file_type", "is_point_file_path", b"is_point_file_path", "list_files_pre_step_configuration", b"list_files_pre_step_configuration", "output_file_type", b"output_file_type", "parameters", b"parameters", "secret_secret", b"secret_secret", "service_account_json_key_secret", b"service_account_json_key_secret", "source_bucket", b"source_bucket", "token_secret", b"token_secret"]) -> None: ...

global___InferenceTaskConfigurationMessage = InferenceTaskConfigurationMessage

class BatchJobParameter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.str
    value: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

global___BatchJobParameter = BatchJobParameter

class CleanInferenceTasksExecutorsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFERENCE_JOB_ID_FIELD_NUMBER: builtins.int
    inference_job_id: builtins.str
    """The id of the inference job"""
    def __init__(
        self,
        *,
        inference_job_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["inference_job_id", b"inference_job_id"]) -> None: ...

global___CleanInferenceTasksExecutorsRequest = CleanInferenceTasksExecutorsRequest

class CleanInferenceTasksExecutorsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    success: builtins.bool
    """Whether the request was successful or not"""
    failure_reason: builtins.str
    """The failure description"""
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
        failure_reason: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "success", b"success"]) -> None: ...

global___CleanInferenceTasksExecutorsResponse = CleanInferenceTasksExecutorsResponse

class CleanInferenceTaskExecutorRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFERENCE_JOB_ID_FIELD_NUMBER: builtins.int
    INFERENCE_TASK_ID_FIELD_NUMBER: builtins.int
    inference_job_id: builtins.str
    """The id of the inference job"""
    inference_task_id: builtins.str
    """The id of the inference task"""
    def __init__(
        self,
        *,
        inference_job_id: builtins.str = ...,
        inference_task_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["inference_job_id", b"inference_job_id", "inference_task_id", b"inference_task_id"]) -> None: ...

global___CleanInferenceTaskExecutorRequest = CleanInferenceTaskExecutorRequest

class CleanInferenceTaskExecutorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    success: builtins.bool
    """Whether the request was successful or not"""
    failure_reason: builtins.str
    """The failure description"""
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
        failure_reason: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "success", b"success"]) -> None: ...

global___CleanInferenceTaskExecutorResponse = CleanInferenceTaskExecutorResponse

class CreateInferenceTaskExecutorResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    success: builtins.bool
    """Whether the request was successful or not"""
    failure_reason: builtins.str
    """The failure description"""
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
        failure_reason: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "success", b"success"]) -> None: ...

global___CreateInferenceTaskExecutorResponse = CreateInferenceTaskExecutorResponse

class StartInferenceJobWarmupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    BRANCH_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    IMAGE_URL_FIELD_NUMBER: builtins.int
    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    EXECUTORS_FIELD_NUMBER: builtins.int
    TIMEOUT_FIELD_NUMBER: builtins.int
    JOB_SIZE_FIELD_NUMBER: builtins.int
    MODEL_UUID_FIELD_NUMBER: builtins.int
    IMAGE_PULL_SECRET_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """Model details"""
    branch_id: builtins.str
    """branch id"""
    build_id: builtins.str
    """Build Id"""
    image_url: builtins.str
    """The image url to use"""
    cpu: builtins.float
    """Cpu usage"""
    memory_amount: builtins.int
    """Memory usage"""
    memory_units: global___MemoryUnitApi.ValueType
    """Units type of memory"""
    executors: builtins.int
    """Number of executors"""
    timeout: builtins.int
    """Warmup timeout in seconds"""
    @property
    def job_size(self) -> global___BatchJobResources: ...
    model_uuid: builtins.str
    image_pull_secret: builtins.str
    """Image pull secret"""
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        branch_id: builtins.str = ...,
        build_id: builtins.str = ...,
        image_url: builtins.str = ...,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnitApi.ValueType = ...,
        executors: builtins.int = ...,
        timeout: builtins.int = ...,
        job_size: global___BatchJobResources | None = ...,
        model_uuid: builtins.str = ...,
        image_pull_secret: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["job_size", b"job_size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["branch_id", b"branch_id", "build_id", b"build_id", "cpu", b"cpu", "executors", b"executors", "image_pull_secret", b"image_pull_secret", "image_url", b"image_url", "job_size", b"job_size", "memory_amount", b"memory_amount", "memory_units", b"memory_units", "model_id", b"model_id", "model_uuid", b"model_uuid", "timeout", b"timeout"]) -> None: ...

global___StartInferenceJobWarmupRequest = StartInferenceJobWarmupRequest

class StartInferenceJobWarmupResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartInferenceJobWarmupResponse = StartInferenceJobWarmupResponse

class CancelInferenceJobWarmupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    BRANCH_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    MODEL_UUID_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    branch_id: builtins.str
    build_id: builtins.str
    model_uuid: builtins.str
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        branch_id: builtins.str = ...,
        build_id: builtins.str = ...,
        model_uuid: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["branch_id", b"branch_id", "build_id", b"build_id", "model_id", b"model_id", "model_uuid", b"model_uuid"]) -> None: ...

global___CancelInferenceJobWarmupRequest = CancelInferenceJobWarmupRequest

class CancelInferenceJobWarmupResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CancelInferenceJobWarmupResponse = CancelInferenceJobWarmupResponse

class BatchJobResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMBER_OF_PODS_FIELD_NUMBER: builtins.int
    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    number_of_pods: builtins.int
    """Number of pods to deploy"""
    cpu: builtins.float
    """Cpu"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnitApi.ValueType
    """Units type of memory"""
    @property
    def gpu_resources(self) -> global___GpuResources:
        """Optional GPU resources for batch"""
    def __init__(
        self,
        *,
        number_of_pods: builtins.int = ...,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnitApi.ValueType = ...,
        gpu_resources: global___GpuResources | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["gpu_resources", b"gpu_resources"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu", b"cpu", "gpu_resources", b"gpu_resources", "memory_amount", b"memory_amount", "memory_units", b"memory_units", "number_of_pods", b"number_of_pods"]) -> None: ...

global___BatchJobResources = BatchJobResources

class GpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GPU_TYPE_FIELD_NUMBER: builtins.int
    GPU_AMOUNT_FIELD_NUMBER: builtins.int
    gpu_type: global___GpuType.ValueType
    """The type of the GPU"""
    gpu_amount: builtins.int
    """Amount of GPUs"""
    def __init__(
        self,
        *,
        gpu_type: global___GpuType.ValueType = ...,
        gpu_amount: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gpu_amount", b"gpu_amount", "gpu_type", b"gpu_type"]) -> None: ...

global___GpuResources = GpuResources



from dataclasses import dataclass
from typing import Dict, List

import grpc
from _qwak_proto.qwak.vectors.v1.vector_pb2 import (
    DoubleVector,
    Property,
    SearchResult,
    StoredVector,
    VectorIdentifier,
)
from _qwak_proto.qwak.vectors.v1.vector_service_pb2 import (
    DeleteVectorsResponse,
    FetchVectorRequest,
    FetchVectorResponse,
    SearchSimilarVectorsRequest,
    SearchSimilarVectorsResponse,
    UpsertVectorsRequest,
    UpsertVectorsResponse,
)
from _qwak_proto.qwak.vectors.v1.vector_service_pb2_grpc import VectorServiceServicer
from numpy import dot
from numpy.linalg import norm


@dataclass
class VectorObject:
    id: str
    vector: List[float]
    property: List[Property]


class VectorServingServiceMock(VectorServiceServicer):
    def __init__(self):
        self._vector_collections: Dict[str, Dict] = dict()

    def reset_vector_store(self) -> None:
        self._vector_collections.clear()

    def get_num_of_vectors(self, collection_name: str) -> int:
        if collection_name not in self._vector_collections:
            return 0

        return len(self._vector_collections[collection_name].values())

    def create_collection(self, collection_name: str) -> None:
        self._vector_collections[collection_name] = dict()

    def get_vector_by_ids(
        self, collection_name: str, vector_ids: List[str]
    ) -> List[dict]:
        if collection_name not in self._vector_collections:
            raise ValueError(f"Collection named {collection_name} doesn't exist")

        collection = self._vector_collections[collection_name]
        return [collection.get(_id) for _id in vector_ids if _id in collection]

    def SearchSimilarVectors(self, request: SearchSimilarVectorsRequest, context):
        if request.collection_name not in self._vector_collections:
            context.set_details(f"Collection named {request.collection} doesn't exist'")
            context.set_code(grpc.StatusCode.NOT_FOUND)
            return

        reference_vector = list(request.reference_vector.element)
        collection_store = self._vector_collections.get(request.collection_name, dict())

        # a naive impl of a "vector similarity" search - compute pairwise cosine distance on
        # the entire set and return top results
        result_set = sorted(
            collection_store.values(),
            key=lambda b: _cos_sim(reference_vector, b["vector"]),
        )[-request.max_results :]

        return SearchSimilarVectorsResponse(
            search_results=[
                SearchResult(
                    id=result["id"] if request.include_id else None,
                    properties=[
                        p
                        for p in result_set[0]["properties"]
                        if p.name in request.properties
                    ],
                    vector=DoubleVector(element=result["vector"])
                    if request.include_vector
                    else None,
                    distance=_cos_sim(reference_vector, result["vector"])
                    if request.include_distance
                    else None,
                )
                for result in result_set
            ]
        )

    def UpsertVectors(self, request: UpsertVectorsRequest, context):
        collection_store = self._vector_collections.get(request.collection_name, dict())
        for stored_vector in request.vector:
            id = stored_vector.vector_identifier.vector_id
            collection_store[id] = {
                "id": id,
                "vector": list(stored_vector.vector.element),
                "properties": stored_vector.properties,
            }

        self._vector_collections[request.collection_name] = collection_store
        return UpsertVectorsResponse()

    def DeleteVectors(self, request, context):
        if request.collection_name not in self._vector_collections:
            context.set_details(
                f"Collection named {request.collection_name} doesn't exist'"
            )
            context.set_code(grpc.StatusCode.NOT_FOUND)
            return

        collection_store = self._vector_collections[request.collection_name]
        ids_in_collection = [
            vector_identifier.vector_id
            for vector_identifier in request.vector_identifiers
            if vector_identifier.vector_id in collection_store
        ]
        for id in ids_in_collection:
            collection_store.pop(id)

        self._vector_collections[request.collection_name] = collection_store
        return DeleteVectorsResponse(num_vectors_deleted=len(ids_in_collection))

    def FetchVector(self, request, context):
        if request.collection_name not in self._vector_collections:
            context.set_details(
                f"Collection named {request.collection_name} doesn't exist'"
            )
            context.set_code(grpc.StatusCode.NOT_FOUND)
            return

        collection_store = self._vector_collections[request.collection_name]
        vector_identifier = request.vector_identifier
        vector_id = vector_identifier.vector_id
        if vector_id not in collection_store:
            context.set_details(f"Vector with id {vector_id} doesn't exist'")
            context.set_code(grpc.StatusCode.NOT_FOUND)
            return

        stored_vector_dict = collection_store[vector_id]
        return FetchVectorResponse(
            vector=StoredVector(
                vector_identifier=request.vector_identifier,
                vector=DoubleVector(element=stored_vector_dict["vector"]),
                properties=stored_vector_dict["properties"],
            )
        )


def _cos_sim(a, b):
    return dot(a, b) / (norm(a) * norm(b))
